# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: connect.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

import librespot.proto.Player_pb2 as player__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='connect.proto',
    package='connectstate',
    syntax='proto3',
    serialized_options=b'\n\030com.spotify.connectstateH\002',
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\rconnect.proto\x12\x0c\x63onnectstate\x1a\x0cplayer.proto\"\x9f\x01\n\rClusterUpdate\x12&\n\x07\x63luster\x18\x01 \x01(\x0b\x32\x15.connectstate.Cluster\x12\x38\n\rupdate_reason\x18\x02 \x01(\x0e\x32!.connectstate.ClusterUpdateReason\x12\x0e\n\x06\x61\x63k_id\x18\x03 \x01(\t\x12\x1c\n\x14\x64\x65vices_that_changed\x18\x04 \x03(\t\"\xa6\x01\n\x06\x44\x65vice\x12-\n\x0b\x64\x65vice_info\x18\x01 \x01(\x0b\x32\x18.connectstate.DeviceInfo\x12/\n\x0cplayer_state\x18\x02 \x01(\x0b\x32\x19.connectstate.PlayerState\x12<\n\x13private_device_info\x18\x03 \x01(\x0b\x32\x1f.connectstate.PrivateDeviceInfo\"\xfa\x01\n\x07\x43luster\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x18\n\x10\x61\x63tive_device_id\x18\x02 \x01(\t\x12/\n\x0cplayer_state\x18\x03 \x01(\x0b\x32\x19.connectstate.PlayerState\x12\x31\n\x06\x64\x65vice\x18\x04 \x03(\x0b\x32!.connectstate.Cluster.DeviceEntry\x12\x15\n\rtransfer_data\x18\x05 \x01(\x0c\x1aG\n\x0b\x44\x65viceEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.connectstate.DeviceInfo:\x02\x38\x01\"\xa1\x03\n\x0fPutStateRequest\x12\x14\n\x0c\x63\x61llback_url\x18\x01 \x01(\t\x12$\n\x06\x64\x65vice\x18\x02 \x01(\x0b\x32\x14.connectstate.Device\x12-\n\x0bmember_type\x18\x03 \x01(\x0e\x32\x18.connectstate.MemberType\x12\x11\n\tis_active\x18\x04 \x01(\x08\x12\x36\n\x10put_state_reason\x18\x05 \x01(\x0e\x32\x1c.connectstate.PutStateReason\x12\x12\n\nmessage_id\x18\x06 \x01(\r\x12&\n\x1elast_command_sent_by_device_id\x18\x07 \x01(\t\x12\x1f\n\x17last_command_message_id\x18\x08 \x01(\r\x12\x1a\n\x12started_playing_at\x18\t \x01(\x04\x12\x1f\n\x17has_been_playing_for_ms\x18\x0b \x01(\x04\x12\x1d\n\x15\x63lient_side_timestamp\x18\x0c \x01(\x04\x12\x1f\n\x17only_write_player_state\x18\r \x01(\x08\"%\n\x11PrivateDeviceInfo\x12\x10\n\x08platform\x18\x01 \x01(\t\"(\n\x10SubscribeRequest\x12\x14\n\x0c\x63\x61llback_url\x18\x01 \x01(\t\"\xc5\x03\n\nDeviceInfo\x12\x10\n\x08\x63\x61n_play\x18\x01 \x01(\x08\x12\x0e\n\x06volume\x18\x02 \x01(\r\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x30\n\x0c\x63\x61pabilities\x18\x04 \x01(\x0b\x32\x1a.connectstate.Capabilities\x12\x1f\n\x17\x64\x65vice_software_version\x18\x06 \x01(\t\x12-\n\x0b\x64\x65vice_type\x18\x07 \x01(\x0e\x32\x18.connectstate.DeviceType\x12\x15\n\rspirc_version\x18\t \x01(\t\x12\x11\n\tdevice_id\x18\n \x01(\t\x12\x1a\n\x12is_private_session\x18\x0b \x01(\x08\x12\x19\n\x11is_social_connect\x18\x0c \x01(\x08\x12\x11\n\tclient_id\x18\r \x01(\t\x12\r\n\x05\x62rand\x18\x0e \x01(\t\x12\r\n\x05model\x18\x0f \x01(\t\x12?\n\x0cmetadata_map\x18\x10 \x03(\x0b\x32).connectstate.DeviceInfo.MetadataMapEntry\x1a\x32\n\x10MetadataMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xcc\x04\n\x0c\x43\x61pabilities\x12\x15\n\rcan_be_player\x18\x02 \x01(\x08\x12\x19\n\x11restrict_to_local\x18\x03 \x01(\x08\x12\x1a\n\x12gaia_eq_connect_id\x18\x05 \x01(\x08\x12\x17\n\x0fsupports_logout\x18\x06 \x01(\x08\x12\x15\n\ris_observable\x18\x07 \x01(\x08\x12\x14\n\x0cvolume_steps\x18\x08 \x01(\x05\x12\x17\n\x0fsupported_types\x18\t \x03(\t\x12\x14\n\x0c\x63ommand_acks\x18\n \x01(\x08\x12\x17\n\x0fsupports_rename\x18\x0b \x01(\x08\x12\x0e\n\x06hidden\x18\x0c \x01(\x08\x12\x16\n\x0e\x64isable_volume\x18\r \x01(\x08\x12\x18\n\x10\x63onnect_disabled\x18\x0e \x01(\x08\x12\x1c\n\x14supports_playlist_v2\x18\x0f \x01(\x08\x12\x17\n\x0fis_controllable\x18\x10 \x01(\x08\x12\"\n\x1asupports_external_episodes\x18\x11 \x01(\x08\x12%\n\x1dsupports_set_backend_metadata\x18\x12 \x01(\x08\x12!\n\x19supports_transfer_command\x18\x13 \x01(\x08\x12 \n\x18supports_command_request\x18\x14 \x01(\x08\x12\x18\n\x10is_voice_enabled\x18\x15 \x01(\x08\x12\x1f\n\x17needs_full_player_state\x18\x16 \x01(\x08\x12\x1c\n\x14supports_gzip_pushes\x18\x17 \x01(\x08\"+\n\x15\x43onnectCommandOptions\x12\x12\n\nmessage_id\x18\x01 \x01(\x05\"M\n\rLogoutCommand\x12<\n\x0f\x63ommand_options\x18\x01 \x01(\x0b\x32#.connectstate.ConnectCommandOptions\"`\n\x10SetVolumeCommand\x12\x0e\n\x06volume\x18\x01 \x01(\x05\x12<\n\x0f\x63ommand_options\x18\x02 \x01(\x0b\x32#.connectstate.ConnectCommandOptions\"`\n\rRenameCommand\x12\x11\n\trename_to\x18\x01 \x01(\t\x12<\n\x0f\x63ommand_options\x18\x02 \x01(\x0b\x32#.connectstate.ConnectCommandOptions\"\x95\x01\n\x19SetBackendMetadataCommand\x12G\n\x08metadata\x18\x01 \x03(\x0b\x32\x35.connectstate.SetBackendMetadataCommand.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01*\xd4\x01\n\x11SendCommandResult\x12\x1f\n\x1bUNKNOWN_SEND_COMMAND_RESULT\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\x14\n\x10\x44\x45VICE_NOT_FOUND\x10\x02\x12\x18\n\x14\x43ONTEXT_PLAYER_ERROR\x10\x03\x12\x16\n\x12\x44\x45VICE_DISAPPEARED\x10\x04\x12\x12\n\x0eUPSTREAM_ERROR\x10\x05\x12#\n\x1f\x44\x45VICE_DOES_NOT_SUPPORT_COMMAND\x10\x06\x12\x10\n\x0cRATE_LIMITED\x10\x07*\xb7\x01\n\x0ePutStateReason\x12\x1c\n\x18UNKNOWN_PUT_STATE_REASON\x10\x00\x12\x0f\n\x0bSPIRC_HELLO\x10\x01\x12\x10\n\x0cSPIRC_NOTIFY\x10\x02\x12\x0e\n\nNEW_DEVICE\x10\x03\x12\x18\n\x14PLAYER_STATE_CHANGED\x10\x04\x12\x12\n\x0eVOLUME_CHANGED\x10\x05\x12\x11\n\rPICKER_OPENED\x10\x06\x12\x13\n\x0f\x42\x45\x43\x41ME_INACTIVE\x10\x07*;\n\nMemberType\x12\x0c\n\x08SPIRC_V2\x10\x00\x12\x0c\n\x08SPIRC_V3\x10\x01\x12\x11\n\rCONNECT_STATE\x10\x02*\x84\x01\n\x13\x43lusterUpdateReason\x12!\n\x1dUNKNOWN_CLUSTER_UPDATE_REASON\x10\x00\x12\x17\n\x13\x44\x45VICES_DISAPPEARED\x10\x01\x12\x18\n\x14\x44\x45VICE_STATE_CHANGED\x10\x02\x12\x17\n\x13NEW_DEVICE_APPEARED\x10\x03*\xa0\x02\n\nDeviceType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08\x43OMPUTER\x10\x01\x12\n\n\x06TABLET\x10\x02\x12\x0e\n\nSMARTPHONE\x10\x03\x12\x0b\n\x07SPEAKER\x10\x04\x12\x06\n\x02TV\x10\x05\x12\x07\n\x03\x41VR\x10\x06\x12\x07\n\x03STB\x10\x07\x12\x10\n\x0c\x41UDIO_DONGLE\x10\x08\x12\x10\n\x0cGAME_CONSOLE\x10\t\x12\x0e\n\nCAST_VIDEO\x10\n\x12\x0e\n\nCAST_AUDIO\x10\x0b\x12\x0e\n\nAUTOMOBILE\x10\x0c\x12\x0e\n\nSMARTWATCH\x10\r\x12\x0e\n\nCHROMEBOOK\x10\x0e\x12\x13\n\x0fUNKNOWN_SPOTIFY\x10\x64\x12\r\n\tCAR_THING\x10\x65\x12\x0c\n\x08OBSERVER\x10\x66\x12\x0e\n\nHOME_THING\x10gB\x1c\n\x18\x63om.spotify.connectstateH\x02\x62\x06proto3',
    dependencies=[
        player__pb2.DESCRIPTOR,
    ])

_SENDCOMMANDRESULT = _descriptor.EnumDescriptor(
    name='SendCommandResult',
    full_name='connectstate.SendCommandResult',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='UNKNOWN_SEND_COMMAND_RESULT',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SUCCESS',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='DEVICE_NOT_FOUND',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CONTEXT_PLAYER_ERROR',
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='DEVICE_DISAPPEARED',
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='UPSTREAM_ERROR',
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='DEVICE_DOES_NOT_SUPPORT_COMMAND',
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='RATE_LIMITED',
            index=7,
            number=7,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=2650,
    serialized_end=2862,
)
_sym_db.RegisterEnumDescriptor(_SENDCOMMANDRESULT)

SendCommandResult = enum_type_wrapper.EnumTypeWrapper(_SENDCOMMANDRESULT)
_PUTSTATEREASON = _descriptor.EnumDescriptor(
    name='PutStateReason',
    full_name='connectstate.PutStateReason',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='UNKNOWN_PUT_STATE_REASON',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SPIRC_HELLO',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SPIRC_NOTIFY',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='NEW_DEVICE',
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='PLAYER_STATE_CHANGED',
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='VOLUME_CHANGED',
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='PICKER_OPENED',
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='BECAME_INACTIVE',
            index=7,
            number=7,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=2865,
    serialized_end=3048,
)
_sym_db.RegisterEnumDescriptor(_PUTSTATEREASON)

PutStateReason = enum_type_wrapper.EnumTypeWrapper(_PUTSTATEREASON)
_MEMBERTYPE = _descriptor.EnumDescriptor(
    name='MemberType',
    full_name='connectstate.MemberType',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='SPIRC_V2',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SPIRC_V3',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CONNECT_STATE',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=3050,
    serialized_end=3109,
)
_sym_db.RegisterEnumDescriptor(_MEMBERTYPE)

MemberType = enum_type_wrapper.EnumTypeWrapper(_MEMBERTYPE)
_CLUSTERUPDATEREASON = _descriptor.EnumDescriptor(
    name='ClusterUpdateReason',
    full_name='connectstate.ClusterUpdateReason',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='UNKNOWN_CLUSTER_UPDATE_REASON',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='DEVICES_DISAPPEARED',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='DEVICE_STATE_CHANGED',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='NEW_DEVICE_APPEARED',
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=3112,
    serialized_end=3244,
)
_sym_db.RegisterEnumDescriptor(_CLUSTERUPDATEREASON)

ClusterUpdateReason = enum_type_wrapper.EnumTypeWrapper(_CLUSTERUPDATEREASON)
_DEVICETYPE = _descriptor.EnumDescriptor(
    name='DeviceType',
    full_name='connectstate.DeviceType',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='UNKNOWN',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='COMPUTER',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='TABLET',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SMARTPHONE',
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SPEAKER',
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='TV',
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='AVR',
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='STB',
            index=7,
            number=7,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='AUDIO_DONGLE',
            index=8,
            number=8,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='GAME_CONSOLE',
            index=9,
            number=9,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CAST_VIDEO',
            index=10,
            number=10,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CAST_AUDIO',
            index=11,
            number=11,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='AUTOMOBILE',
            index=12,
            number=12,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='SMARTWATCH',
            index=13,
            number=13,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CHROMEBOOK',
            index=14,
            number=14,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='UNKNOWN_SPOTIFY',
            index=15,
            number=100,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='CAR_THING',
            index=16,
            number=101,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='OBSERVER',
            index=17,
            number=102,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name='HOME_THING',
            index=18,
            number=103,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=3247,
    serialized_end=3535,
)
_sym_db.RegisterEnumDescriptor(_DEVICETYPE)

DeviceType = enum_type_wrapper.EnumTypeWrapper(_DEVICETYPE)
UNKNOWN_SEND_COMMAND_RESULT = 0
SUCCESS = 1
DEVICE_NOT_FOUND = 2
CONTEXT_PLAYER_ERROR = 3
DEVICE_DISAPPEARED = 4
UPSTREAM_ERROR = 5
DEVICE_DOES_NOT_SUPPORT_COMMAND = 6
RATE_LIMITED = 7
UNKNOWN_PUT_STATE_REASON = 0
SPIRC_HELLO = 1
SPIRC_NOTIFY = 2
NEW_DEVICE = 3
PLAYER_STATE_CHANGED = 4
VOLUME_CHANGED = 5
PICKER_OPENED = 6
BECAME_INACTIVE = 7
SPIRC_V2 = 0
SPIRC_V3 = 1
CONNECT_STATE = 2
UNKNOWN_CLUSTER_UPDATE_REASON = 0
DEVICES_DISAPPEARED = 1
DEVICE_STATE_CHANGED = 2
NEW_DEVICE_APPEARED = 3
UNKNOWN = 0
COMPUTER = 1
TABLET = 2
SMARTPHONE = 3
SPEAKER = 4
TV = 5
AVR = 6
STB = 7
AUDIO_DONGLE = 8
GAME_CONSOLE = 9
CAST_VIDEO = 10
CAST_AUDIO = 11
AUTOMOBILE = 12
SMARTWATCH = 13
CHROMEBOOK = 14
UNKNOWN_SPOTIFY = 100
CAR_THING = 101
OBSERVER = 102
HOME_THING = 103

_CLUSTERUPDATE = _descriptor.Descriptor(
    name='ClusterUpdate',
    full_name='connectstate.ClusterUpdate',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='cluster',
            full_name='connectstate.ClusterUpdate.cluster',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='update_reason',
            full_name='connectstate.ClusterUpdate.update_reason',
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='ack_id',
            full_name='connectstate.ClusterUpdate.ack_id',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='devices_that_changed',
            full_name='connectstate.ClusterUpdate.devices_that_changed',
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=46,
    serialized_end=205,
)

_DEVICE = _descriptor.Descriptor(
    name='Device',
    full_name='connectstate.Device',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='device_info',
            full_name='connectstate.Device.device_info',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='player_state',
            full_name='connectstate.Device.player_state',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='private_device_info',
            full_name='connectstate.Device.private_device_info',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=208,
    serialized_end=374,
)

_CLUSTER_DEVICEENTRY = _descriptor.Descriptor(
    name='DeviceEntry',
    full_name='connectstate.Cluster.DeviceEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='connectstate.Cluster.DeviceEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='connectstate.Cluster.DeviceEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=556,
    serialized_end=627,
)

_CLUSTER = _descriptor.Descriptor(
    name='Cluster',
    full_name='connectstate.Cluster',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='timestamp',
            full_name='connectstate.Cluster.timestamp',
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='active_device_id',
            full_name='connectstate.Cluster.active_device_id',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='player_state',
            full_name='connectstate.Cluster.player_state',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='device',
            full_name='connectstate.Cluster.device',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='transfer_data',
            full_name='connectstate.Cluster.transfer_data',
            index=4,
            number=5,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[
        _CLUSTER_DEVICEENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=377,
    serialized_end=627,
)

_PUTSTATEREQUEST = _descriptor.Descriptor(
    name='PutStateRequest',
    full_name='connectstate.PutStateRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='callback_url',
            full_name='connectstate.PutStateRequest.callback_url',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='device',
            full_name='connectstate.PutStateRequest.device',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='member_type',
            full_name='connectstate.PutStateRequest.member_type',
            index=2,
            number=3,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_active',
            full_name='connectstate.PutStateRequest.is_active',
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='put_state_reason',
            full_name='connectstate.PutStateRequest.put_state_reason',
            index=4,
            number=5,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='message_id',
            full_name='connectstate.PutStateRequest.message_id',
            index=5,
            number=6,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='last_command_sent_by_device_id',
            full_name=
            'connectstate.PutStateRequest.last_command_sent_by_device_id',
            index=6,
            number=7,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='last_command_message_id',
            full_name='connectstate.PutStateRequest.last_command_message_id',
            index=7,
            number=8,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='started_playing_at',
            full_name='connectstate.PutStateRequest.started_playing_at',
            index=8,
            number=9,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='has_been_playing_for_ms',
            full_name='connectstate.PutStateRequest.has_been_playing_for_ms',
            index=9,
            number=11,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='client_side_timestamp',
            full_name='connectstate.PutStateRequest.client_side_timestamp',
            index=10,
            number=12,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='only_write_player_state',
            full_name='connectstate.PutStateRequest.only_write_player_state',
            index=11,
            number=13,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=630,
    serialized_end=1047,
)

_PRIVATEDEVICEINFO = _descriptor.Descriptor(
    name='PrivateDeviceInfo',
    full_name='connectstate.PrivateDeviceInfo',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='platform',
            full_name='connectstate.PrivateDeviceInfo.platform',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1049,
    serialized_end=1086,
)

_SUBSCRIBEREQUEST = _descriptor.Descriptor(
    name='SubscribeRequest',
    full_name='connectstate.SubscribeRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='callback_url',
            full_name='connectstate.SubscribeRequest.callback_url',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1088,
    serialized_end=1128,
)

_DEVICEINFO_METADATAMAPENTRY = _descriptor.Descriptor(
    name='MetadataMapEntry',
    full_name='connectstate.DeviceInfo.MetadataMapEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='connectstate.DeviceInfo.MetadataMapEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='connectstate.DeviceInfo.MetadataMapEntry.value',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1534,
    serialized_end=1584,
)

_DEVICEINFO = _descriptor.Descriptor(
    name='DeviceInfo',
    full_name='connectstate.DeviceInfo',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='can_play',
            full_name='connectstate.DeviceInfo.can_play',
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='volume',
            full_name='connectstate.DeviceInfo.volume',
            index=1,
            number=2,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='name',
            full_name='connectstate.DeviceInfo.name',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='capabilities',
            full_name='connectstate.DeviceInfo.capabilities',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='device_software_version',
            full_name='connectstate.DeviceInfo.device_software_version',
            index=4,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='device_type',
            full_name='connectstate.DeviceInfo.device_type',
            index=5,
            number=7,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='spirc_version',
            full_name='connectstate.DeviceInfo.spirc_version',
            index=6,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='device_id',
            full_name='connectstate.DeviceInfo.device_id',
            index=7,
            number=10,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_private_session',
            full_name='connectstate.DeviceInfo.is_private_session',
            index=8,
            number=11,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_social_connect',
            full_name='connectstate.DeviceInfo.is_social_connect',
            index=9,
            number=12,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='client_id',
            full_name='connectstate.DeviceInfo.client_id',
            index=10,
            number=13,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='brand',
            full_name='connectstate.DeviceInfo.brand',
            index=11,
            number=14,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='model',
            full_name='connectstate.DeviceInfo.model',
            index=12,
            number=15,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='metadata_map',
            full_name='connectstate.DeviceInfo.metadata_map',
            index=13,
            number=16,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[
        _DEVICEINFO_METADATAMAPENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1131,
    serialized_end=1584,
)

_CAPABILITIES = _descriptor.Descriptor(
    name='Capabilities',
    full_name='connectstate.Capabilities',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='can_be_player',
            full_name='connectstate.Capabilities.can_be_player',
            index=0,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='restrict_to_local',
            full_name='connectstate.Capabilities.restrict_to_local',
            index=1,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='gaia_eq_connect_id',
            full_name='connectstate.Capabilities.gaia_eq_connect_id',
            index=2,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_logout',
            full_name='connectstate.Capabilities.supports_logout',
            index=3,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_observable',
            full_name='connectstate.Capabilities.is_observable',
            index=4,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='volume_steps',
            full_name='connectstate.Capabilities.volume_steps',
            index=5,
            number=8,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supported_types',
            full_name='connectstate.Capabilities.supported_types',
            index=6,
            number=9,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='command_acks',
            full_name='connectstate.Capabilities.command_acks',
            index=7,
            number=10,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_rename',
            full_name='connectstate.Capabilities.supports_rename',
            index=8,
            number=11,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='hidden',
            full_name='connectstate.Capabilities.hidden',
            index=9,
            number=12,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='disable_volume',
            full_name='connectstate.Capabilities.disable_volume',
            index=10,
            number=13,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='connect_disabled',
            full_name='connectstate.Capabilities.connect_disabled',
            index=11,
            number=14,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_playlist_v2',
            full_name='connectstate.Capabilities.supports_playlist_v2',
            index=12,
            number=15,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_controllable',
            full_name='connectstate.Capabilities.is_controllable',
            index=13,
            number=16,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_external_episodes',
            full_name='connectstate.Capabilities.supports_external_episodes',
            index=14,
            number=17,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_set_backend_metadata',
            full_name='connectstate.Capabilities.supports_set_backend_metadata',
            index=15,
            number=18,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_transfer_command',
            full_name='connectstate.Capabilities.supports_transfer_command',
            index=16,
            number=19,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_command_request',
            full_name='connectstate.Capabilities.supports_command_request',
            index=17,
            number=20,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='is_voice_enabled',
            full_name='connectstate.Capabilities.is_voice_enabled',
            index=18,
            number=21,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='needs_full_player_state',
            full_name='connectstate.Capabilities.needs_full_player_state',
            index=19,
            number=22,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='supports_gzip_pushes',
            full_name='connectstate.Capabilities.supports_gzip_pushes',
            index=20,
            number=23,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1587,
    serialized_end=2175,
)

_CONNECTCOMMANDOPTIONS = _descriptor.Descriptor(
    name='ConnectCommandOptions',
    full_name='connectstate.ConnectCommandOptions',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='message_id',
            full_name='connectstate.ConnectCommandOptions.message_id',
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2177,
    serialized_end=2220,
)

_LOGOUTCOMMAND = _descriptor.Descriptor(
    name='LogoutCommand',
    full_name='connectstate.LogoutCommand',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='command_options',
            full_name='connectstate.LogoutCommand.command_options',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2222,
    serialized_end=2299,
)

_SETVOLUMECOMMAND = _descriptor.Descriptor(
    name='SetVolumeCommand',
    full_name='connectstate.SetVolumeCommand',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='volume',
            full_name='connectstate.SetVolumeCommand.volume',
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='command_options',
            full_name='connectstate.SetVolumeCommand.command_options',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2301,
    serialized_end=2397,
)

_RENAMECOMMAND = _descriptor.Descriptor(
    name='RenameCommand',
    full_name='connectstate.RenameCommand',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='rename_to',
            full_name='connectstate.RenameCommand.rename_to',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='command_options',
            full_name='connectstate.RenameCommand.command_options',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2399,
    serialized_end=2495,
)

_SETBACKENDMETADATACOMMAND_METADATAENTRY = _descriptor.Descriptor(
    name='MetadataEntry',
    full_name='connectstate.SetBackendMetadataCommand.MetadataEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name=
            'connectstate.SetBackendMetadataCommand.MetadataEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
        _descriptor.FieldDescriptor(
            name='value',
            full_name=
            'connectstate.SetBackendMetadataCommand.MetadataEntry.value',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2600,
    serialized_end=2647,
)

_SETBACKENDMETADATACOMMAND = _descriptor.Descriptor(
    name='SetBackendMetadataCommand',
    full_name='connectstate.SetBackendMetadataCommand',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='metadata',
            full_name='connectstate.SetBackendMetadataCommand.metadata',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key),
    ],
    extensions=[],
    nested_types=[
        _SETBACKENDMETADATACOMMAND_METADATAENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2498,
    serialized_end=2647,
)

_CLUSTERUPDATE.fields_by_name['cluster'].message_type = _CLUSTER
_CLUSTERUPDATE.fields_by_name['update_reason'].enum_type = _CLUSTERUPDATEREASON
_DEVICE.fields_by_name['device_info'].message_type = _DEVICEINFO
_DEVICE.fields_by_name['player_state'].message_type = player__pb2._PLAYERSTATE
_DEVICE.fields_by_name['private_device_info'].message_type = _PRIVATEDEVICEINFO
_CLUSTER_DEVICEENTRY.fields_by_name['value'].message_type = _DEVICEINFO
_CLUSTER_DEVICEENTRY.containing_type = _CLUSTER
_CLUSTER.fields_by_name['player_state'].message_type = player__pb2._PLAYERSTATE
_CLUSTER.fields_by_name['device'].message_type = _CLUSTER_DEVICEENTRY
_PUTSTATEREQUEST.fields_by_name['device'].message_type = _DEVICE
_PUTSTATEREQUEST.fields_by_name['member_type'].enum_type = _MEMBERTYPE
_PUTSTATEREQUEST.fields_by_name['put_state_reason'].enum_type = _PUTSTATEREASON
_DEVICEINFO_METADATAMAPENTRY.containing_type = _DEVICEINFO
_DEVICEINFO.fields_by_name['capabilities'].message_type = _CAPABILITIES
_DEVICEINFO.fields_by_name['device_type'].enum_type = _DEVICETYPE
_DEVICEINFO.fields_by_name[
    'metadata_map'].message_type = _DEVICEINFO_METADATAMAPENTRY
_LOGOUTCOMMAND.fields_by_name[
    'command_options'].message_type = _CONNECTCOMMANDOPTIONS
_SETVOLUMECOMMAND.fields_by_name[
    'command_options'].message_type = _CONNECTCOMMANDOPTIONS
_RENAMECOMMAND.fields_by_name[
    'command_options'].message_type = _CONNECTCOMMANDOPTIONS
_SETBACKENDMETADATACOMMAND_METADATAENTRY.containing_type = _SETBACKENDMETADATACOMMAND
_SETBACKENDMETADATACOMMAND.fields_by_name[
    'metadata'].message_type = _SETBACKENDMETADATACOMMAND_METADATAENTRY
DESCRIPTOR.message_types_by_name['ClusterUpdate'] = _CLUSTERUPDATE
DESCRIPTOR.message_types_by_name['Device'] = _DEVICE
DESCRIPTOR.message_types_by_name['Cluster'] = _CLUSTER
DESCRIPTOR.message_types_by_name['PutStateRequest'] = _PUTSTATEREQUEST
DESCRIPTOR.message_types_by_name['PrivateDeviceInfo'] = _PRIVATEDEVICEINFO
DESCRIPTOR.message_types_by_name['SubscribeRequest'] = _SUBSCRIBEREQUEST
DESCRIPTOR.message_types_by_name['DeviceInfo'] = _DEVICEINFO
DESCRIPTOR.message_types_by_name['Capabilities'] = _CAPABILITIES
DESCRIPTOR.message_types_by_name[
    'ConnectCommandOptions'] = _CONNECTCOMMANDOPTIONS
DESCRIPTOR.message_types_by_name['LogoutCommand'] = _LOGOUTCOMMAND
DESCRIPTOR.message_types_by_name['SetVolumeCommand'] = _SETVOLUMECOMMAND
DESCRIPTOR.message_types_by_name['RenameCommand'] = _RENAMECOMMAND
DESCRIPTOR.message_types_by_name[
    'SetBackendMetadataCommand'] = _SETBACKENDMETADATACOMMAND
DESCRIPTOR.enum_types_by_name['SendCommandResult'] = _SENDCOMMANDRESULT
DESCRIPTOR.enum_types_by_name['PutStateReason'] = _PUTSTATEREASON
DESCRIPTOR.enum_types_by_name['MemberType'] = _MEMBERTYPE
DESCRIPTOR.enum_types_by_name['ClusterUpdateReason'] = _CLUSTERUPDATEREASON
DESCRIPTOR.enum_types_by_name['DeviceType'] = _DEVICETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ClusterUpdate = _reflection.GeneratedProtocolMessageType(
    'ClusterUpdate',
    (_message.Message, ),
    {
        'DESCRIPTOR': _CLUSTERUPDATE,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.ClusterUpdate)
    })
_sym_db.RegisterMessage(ClusterUpdate)

Device = _reflection.GeneratedProtocolMessageType(
    'Device',
    (_message.Message, ),
    {
        'DESCRIPTOR': _DEVICE,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.Device)
    })
_sym_db.RegisterMessage(Device)

Cluster = _reflection.GeneratedProtocolMessageType(
    'Cluster',
    (_message.Message, ),
    {
        'DeviceEntry':
        _reflection.GeneratedProtocolMessageType(
            'DeviceEntry',
            (_message.Message, ),
            {
                'DESCRIPTOR': _CLUSTER_DEVICEENTRY,
                '__module__': 'connect_pb2'
                # @@protoc_insertion_point(class_scope:connectstate.Cluster.DeviceEntry)
            }),
        'DESCRIPTOR':
        _CLUSTER,
        '__module__':
        'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.Cluster)
    })
_sym_db.RegisterMessage(Cluster)
_sym_db.RegisterMessage(Cluster.DeviceEntry)

PutStateRequest = _reflection.GeneratedProtocolMessageType(
    'PutStateRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _PUTSTATEREQUEST,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.PutStateRequest)
    })
_sym_db.RegisterMessage(PutStateRequest)

PrivateDeviceInfo = _reflection.GeneratedProtocolMessageType(
    'PrivateDeviceInfo',
    (_message.Message, ),
    {
        'DESCRIPTOR': _PRIVATEDEVICEINFO,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.PrivateDeviceInfo)
    })
_sym_db.RegisterMessage(PrivateDeviceInfo)

SubscribeRequest = _reflection.GeneratedProtocolMessageType(
    'SubscribeRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _SUBSCRIBEREQUEST,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.SubscribeRequest)
    })
_sym_db.RegisterMessage(SubscribeRequest)

DeviceInfo = _reflection.GeneratedProtocolMessageType(
    'DeviceInfo',
    (_message.Message, ),
    {
        'MetadataMapEntry':
        _reflection.GeneratedProtocolMessageType(
            'MetadataMapEntry',
            (_message.Message, ),
            {
                'DESCRIPTOR': _DEVICEINFO_METADATAMAPENTRY,
                '__module__': 'connect_pb2'
                # @@protoc_insertion_point(class_scope:connectstate.DeviceInfo.MetadataMapEntry)
            }),
        'DESCRIPTOR':
        _DEVICEINFO,
        '__module__':
        'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.DeviceInfo)
    })
_sym_db.RegisterMessage(DeviceInfo)
_sym_db.RegisterMessage(DeviceInfo.MetadataMapEntry)

Capabilities = _reflection.GeneratedProtocolMessageType(
    'Capabilities',
    (_message.Message, ),
    {
        'DESCRIPTOR': _CAPABILITIES,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.Capabilities)
    })
_sym_db.RegisterMessage(Capabilities)

ConnectCommandOptions = _reflection.GeneratedProtocolMessageType(
    'ConnectCommandOptions',
    (_message.Message, ),
    {
        'DESCRIPTOR': _CONNECTCOMMANDOPTIONS,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.ConnectCommandOptions)
    })
_sym_db.RegisterMessage(ConnectCommandOptions)

LogoutCommand = _reflection.GeneratedProtocolMessageType(
    'LogoutCommand',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LOGOUTCOMMAND,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.LogoutCommand)
    })
_sym_db.RegisterMessage(LogoutCommand)

SetVolumeCommand = _reflection.GeneratedProtocolMessageType(
    'SetVolumeCommand',
    (_message.Message, ),
    {
        'DESCRIPTOR': _SETVOLUMECOMMAND,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.SetVolumeCommand)
    })
_sym_db.RegisterMessage(SetVolumeCommand)

RenameCommand = _reflection.GeneratedProtocolMessageType(
    'RenameCommand',
    (_message.Message, ),
    {
        'DESCRIPTOR': _RENAMECOMMAND,
        '__module__': 'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.RenameCommand)
    })
_sym_db.RegisterMessage(RenameCommand)

SetBackendMetadataCommand = _reflection.GeneratedProtocolMessageType(
    'SetBackendMetadataCommand',
    (_message.Message, ),
    {
        'MetadataEntry':
        _reflection.GeneratedProtocolMessageType(
            'MetadataEntry',
            (_message.Message, ),
            {
                'DESCRIPTOR': _SETBACKENDMETADATACOMMAND_METADATAENTRY,
                '__module__': 'connect_pb2'
                # @@protoc_insertion_point(class_scope:connectstate.SetBackendMetadataCommand.MetadataEntry)
            }),
        'DESCRIPTOR':
        _SETBACKENDMETADATACOMMAND,
        '__module__':
        'connect_pb2'
        # @@protoc_insertion_point(class_scope:connectstate.SetBackendMetadataCommand)
    })
_sym_db.RegisterMessage(SetBackendMetadataCommand)
_sym_db.RegisterMessage(SetBackendMetadataCommand.MetadataEntry)

DESCRIPTOR._options = None
_CLUSTER_DEVICEENTRY._options = None
_DEVICEINFO_METADATAMAPENTRY._options = None
_SETBACKENDMETADATACOMMAND_METADATAENTRY._options = None
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from librespot.audio import SuperAudioFormat
from librespot.proto import Metadata_pb2 as Metadata
from librespot.proto.Metadata_pb2 import AudioFile
from librespot.structure import AudioQualityPicker
import enum
import logging
import typing


class AudioQuality(enum.Enum):
    NORMAL = 0x00
    HIGH = 0x01
    VERY_HIGH = 0x02

    @staticmethod
    def get_quality(audio_format: AudioFile.Format) -> AudioQuality:
        if audio_format in [
                AudioFile.MP3_96,
                AudioFile.OGG_VORBIS_96,
                AudioFile.AAC_24_NORM,
        ]:
            return AudioQuality.NORMAL
        if audio_format in [
                AudioFile.MP3_160,
                AudioFile.MP3_160_ENC,
                AudioFile.OGG_VORBIS_160,
                AudioFile.AAC_24,
        ]:
            return AudioQuality.HIGH
        if audio_format in [
                AudioFile.MP3_320,
                AudioFile.MP3_256,
                AudioFile.OGG_VORBIS_320,
                AudioFile.AAC_48,
        ]:
            return AudioQuality.VERY_HIGH
        raise RuntimeError("Unknown format: {}".format(format))

    def get_matches(self,
                    files: typing.List[AudioFile]) -> typing.List[AudioFile]:
        file_list = []
        for file in files:
            if hasattr(file, "format") and AudioQuality.get_quality(
                    file.format) == self:
                file_list.append(file)
        return file_list


class VorbisOnlyAudioQuality(AudioQualityPicker):
    logger = logging.getLogger("Librespot:Player:VorbisOnlyAudioQuality")
    preferred: AudioQuality

    def __init__(self, preferred: AudioQuality):
        self.preferred = preferred

    @staticmethod
    def get_vorbis_file(files: typing.List[Metadata.AudioFile]):
        for file in files:
            if file.HasField("format") and SuperAudioFormat.get(
                    file.format) == SuperAudioFormat.VORBIS:
                return file
        return None

    def get_file(self, files: typing.List[Metadata.AudioFile]):
        matches: typing.List[Metadata.AudioFile] = self.preferred.get_matches(
            files)
        vorbis: Metadata.AudioFile = VorbisOnlyAudioQuality.get_vorbis_file(
            matches)
        if vorbis is None:
            vorbis: Metadata.AudioFile = VorbisOnlyAudioQuality.get_vorbis_file(
                files)
            if vorbis is not None:
                self.logger.warning(
                    "Using {} because preferred {} couldn't be found.".format(
                        Metadata.AudioFile.Format.Name(vorbis.format),
                        self.preferred))
            else:
                self.logger.fatal(
                    "Couldn't find any Vorbis file, available: {}")
        return vorbis



from Cryptodome import Random
import binascii
import math


def bytes_to_hex(buffer: bytes) -> str:
    """
    Convert bytes to hex
    Args:
        buffer: Bytes to convert
    Returns:
        hex
    """
    return binascii.hexlify(buffer).decode()


def hex_to_bytes(s: str) -> bytes:
    return binascii.unhexlify(s)


def int_to_bytes(i: int):
    """
    Convert an integer to a byte(s)
    Args:
        i: Integer to convert
    Returns:
        bytes
    """
    width = i.bit_length()
    width += 8 - ((width % 8) or 8)
    fmt = '%%0%dx' % (width // 4)
    return b"\x00" if i == 0 else binascii.unhexlify(fmt % i)


def random_hex_string(length: int):
    buffer = Random.get_random_bytes(int(length / 2))
    return bytes_to_hex(buffer)


class Base62:
    standard_base = 256
    target_base = 62
    alphabet: bytes
    lookup: bytearray

    def __init__(self, alphabet: bytes):
        self.alphabet = alphabet
        self.create_lookup_table()

    @staticmethod
    def create_instance_with_inverted_character_set():
        return Base62(Base62.CharacterSets.inverted)

    def encode(self, message: bytes, length: int = -1):
        indices = self.convert(message, self.standard_base, self.target_base,
                               length)
        return self.translate(indices, self.alphabet)

    def decode(self, encoded: bytes, length: int = -1):
        prepared = self.translate(encoded, self.lookup)
        return self.convert(prepared, self.target_base, self.standard_base,
                            length)

    def translate(self, indices: bytes, dictionary: bytes):
        translation = bytearray(len(indices))
        for i in range(len(indices)):
            translation[i] = dictionary[int.from_bytes(bytes([indices[i]]), "big")]
        return translation

    def convert(self, message: bytes, source_base: int, target_base: int,
                length: int):
        estimated_length = self.estimate_output_length(
            len(message), source_base, target_base) if length == -1 else length
        out = b""
        source = message
        while len(source) > 0:
            quotient = b""
            remainder = 0
            for b in source:
                accumulator = int(b & 0xff) + remainder * source_base
                digit = int(
                    (accumulator - (accumulator % target_base)) / target_base)
                remainder = int(accumulator % target_base)
                if len(quotient) > 0 or digit > 0:
                    quotient += bytes([digit])
            out += bytes([remainder])
            source = quotient
        if len(out) < estimated_length:
            size = len(out)
            for _ in range(estimated_length - size):
                out += bytes([0])
            return self.reverse(out)
        if len(out) > estimated_length:
            return self.reverse(out[:estimated_length])
        return self.reverse(out)

    def estimate_output_length(self, input_length: int, source_base: int,
                               target_base: int):
        return int(
            math.ceil((math.log(source_base) / math.log(target_base)) *
                      input_length))

    def reverse(self, arr: bytes):
        length = len(arr)
        reversed_arr = bytearray(length)
        for i in range(length):
            reversed_arr[length - i - 1] = arr[i]
        return bytes(reversed_arr)

    def create_lookup_table(self):
        self.lookup = bytearray(256)
        for i in range(len(self.alphabet)):
            self.lookup[self.alphabet[i]] = i & 0xff

    class CharacterSets:
        gmp = b'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
        inverted = b'0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: spotify/login5/v3/identifiers/identifiers.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name="spotify/login5/v3/identifiers/identifiers.proto",
    package="spotify.login5.v3.identifiers",
    syntax="proto3",
    serialized_options=b"\n\024com.spotify.login5v3",
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n/spotify/login5/v3/identifiers/identifiers.proto\x12\x1dspotify.login5.v3.identifiers"U\n\x0bPhoneNumber\x12\x0e\n\x06number\x18\x01 \x01(\t\x12\x18\n\x10iso_country_code\x18\x02 \x01(\t\x12\x1c\n\x14\x63ountry_calling_code\x18\x03 \x01(\tB\x16\n\x14\x63om.spotify.login5v3b\x06proto3',
)

_PHONENUMBER = _descriptor.Descriptor(
    name="PhoneNumber",
    full_name="spotify.login5.v3.identifiers.PhoneNumber",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="number",
            full_name="spotify.login5.v3.identifiers.PhoneNumber.number",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="iso_country_code",
            full_name=
            "spotify.login5.v3.identifiers.PhoneNumber.iso_country_code",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="country_calling_code",
            full_name=
            "spotify.login5.v3.identifiers.PhoneNumber.country_calling_code",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=82,
    serialized_end=167,
)

DESCRIPTOR.message_types_by_name["PhoneNumber"] = _PHONENUMBER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PhoneNumber = _reflection.GeneratedProtocolMessageType(
    "PhoneNumber",
    (_message.Message, ),
    {
        "DESCRIPTOR": _PHONENUMBER,
        "__module__": "spotify.login5.v3.identifiers.identifiers_pb2"
        # @@protoc_insertion_point(class_scope:spotify.login5.v3.identifiers.PhoneNumber)
    },
)
_sym_db.RegisterMessage(PhoneNumber)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from librespot import Version
from librespot.audio.decoders import AudioQuality
from librespot.core import Session
from librespot.mercury import MercuryRequests
from librespot.proto import Connect_pb2 as Connect
from librespot.structure import Closeable, MessageListener, RequestListener
import concurrent.futures
import logging
import typing
import urllib.parse


class DeviceStateHandler(Closeable, MessageListener, RequestListener):
    logger = logging.getLogger("Librespot:DeviceStateHandler")
    __closing = False
    __connection_id: str = None
    __device_info: Connect.DeviceInfo
    __put_state: Connect.PutStateRequest
    __put_state_worker = concurrent.futures.ThreadPoolExecutor()
    __session: Session

    def __init__(self, session: Session, conf: PlayerConfiguration):
        self.__session = session
        self.__device_info = self.initialize_device_info(session, conf)
        self.__put_state = Connect.PutStateRequest(
            device=Connect.Device(device_info=self.__device_info, ),
            member_type=Connect.MemberType.CONNECT_STATE,
        )
        self.__session.dealer().add_message_listener(self, [
            "hm://pusher/v1/connections/",
            "hm://connect-state/v1/connect/volume",
            "hm://connect-state/v1/cluster"
        ])
        self.__session.dealer().add_request_listener(self,
                                                     "hm://connect-state/v1/")

    def close(self) -> None:
        self.__closing = True
        self.__session.dealer().remove_message_listener(self)
        self.__session.dealer().remove_request_listener(self)
        self.__put_state_worker.shutdown()

    def initialize_device_info(self, session: Session,
                               conf: PlayerConfiguration) -> Connect.Device:
        return Connect.DeviceInfo(
            can_play=True,
            capabilities=Connect.Capabilities(
                can_be_player=True,
                command_acks=True,
                is_controllable=True,
                is_observable=True,
                gaia_eq_connect_id=True,
                needs_full_player_state=False,
                supported_types=["audio/episode", "audio/track"],
                supports_command_request=True,
                supports_gzip_pushes=True,
                supports_logout=True,
                supports_playlist_v2=True,
                supports_rename=False,
                supports_transfer_command=True,
                volume_steps=conf.volume_steps,
            ),
            client_id=MercuryRequests.keymaster_client_id,
            device_id=session.device_id(),
            device_software_version=Version.version_string(),
            device_type=session.device_type(),
            name=session.device_name(),
            spirc_version="3.2.6",
            volume=conf.initial_volume,
        )

    def put_connect_state(self, request: Connect.PutStateRequest):
        self.__session.api().put_connect_state(self.__connection_id, request)
        self.logger.info("Put state. [ts: {}, connId: {}, reason: {}]".format(
            request.client_side_timestamp, self.__connection_id,
            request.put_state_reason))

    def update_connection_id(self, newer: str) -> None:
        newer = urllib.parse.unquote(newer)
        if self.__connection_id is None or self.__connection_id != newer:
            self.__connection_id = newer
            self.logger.debug(
                "Updated Spotify-Connection-Id: {}".format(newer))


class Player:
    volume_max = 65536
    __conf: PlayerConfiguration
    __session: Session
    __state: StateWrapper

    def __init__(self, conf: PlayerConfiguration, session: Session):
        self.__conf = conf
        self.__session = session

    def init_state(self) -> None:
        self.__state = StateWrapper(self.__session, self, self.__conf)


class PlayerConfiguration:
    # Audio
    preferred_quality: AudioQuality
    enable_normalisation: bool
    normalisation_pregain: float
    autoplay_enabled: bool
    crossfade_duration: int
    preload_enabled: bool

    # Volume
    initial_volume: int
    volume_steps: int

    def __init__(
        self,
        preferred_quality: AudioQuality,
        enable_normalisation: bool,
        normalisation_pregain: float,
        autoplay_enabled: bool,
        crossfade_duration: int,
        preload_enabled: bool,
        initial_volume: int,
        volume_steps: int,
    ):
        self.preferred_quality = preferred_quality
        self.enable_normalisation = enable_normalisation
        self.normalisation_pregain = normalisation_pregain
        self.autoplay_enabled = autoplay_enabled
        self.crossfade_duration = crossfade_duration
        self.preload_enabled = preload_enabled
        self.initial_volume = initial_volume
        self.volume_steps = volume_steps

    class Builder:
        preferred_quality: AudioQuality = AudioQuality.NORMAL
        enable_normalisation: bool = True
        normalisation_pregain: float = 3.0
        autoplay_enabled: bool = True
        crossfade_duration: int = 0
        preload_enabled: bool = True

        # Volume
        initial_volume: int = 65536
        volume_steps: int = 64

        def set_preferred_quality(
                self, preferred_quality: AudioQuality) -> __class__:
            self.preferred_quality = preferred_quality
            return self

        def set_enable_normalisation(self,
                                     enable_normalisation: bool) -> __class__:
            self.enable_normalisation = enable_normalisation
            return self

        def set_normalisation_pregain(
                self, normalisation_pregain: float) -> __class__:
            self.normalisation_pregain = normalisation_pregain
            return self

        def set_autoplay_enabled(self, autoplay_enabled: bool) -> __class__:
            self.autoplay_enabled = autoplay_enabled
            return self

        def set_crossfade_duration(self, crossfade_duration: int) -> __class__:
            self.crossfade_duration = crossfade_duration
            return self

        def set_preload_enabled(self, preload_enabled: bool) -> __class__:
            self.preload_enabled = preload_enabled
            return self

        def build(self) -> PlayerConfiguration:
            return PlayerConfiguration(
                self.preferred_quality,
                self.enable_normalisation,
                self.normalisation_pregain,
                self.autoplay_enabled,
                self.crossfade_duration,
                self.preload_enabled,
                self.initial_volume,
                self.volume_steps,
            )


class StateWrapper(MessageListener):
    __conf: PlayerConfiguration
    __device: DeviceStateHandler
    __player: Player
    __session: Session

    def __init__(self, session: Session, player: Player,
                 conf: PlayerConfiguration):
        self.__session = session
        self.__player = player
        self.__device = DeviceStateHandler(session, conf)
        self.__conf = conf
        session.dealer().add_message_listener(self, [
            "spotify:user:attributes:update", "hm://playlist/",
            "hm://collection/collection/" + session.username() + "/json"
        ])

    def on_message(self, uri: str, headers: typing.Dict[str, str],
                   payload: bytes):
        pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transfer_state.proto
"""Generated protocol buffer code."""
import ContextPlayerOptions_pb2 as context__player__options__pb2
import Playback_pb2 as playback__pb2
import Queue_pb2 as queue__pb2
import Session_pb2 as session__pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name="transfer_state.proto",
    package="spotify.player.proto.transfer",
    syntax="proto2",
    serialized_options=b"\n\024com.spotify.transferH\002",
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b"\n\x14transfer_state.proto\x12\x1dspotify.player.proto.transfer\x1a\x1c\x63ontext_player_options.proto\x1a\x0eplayback.proto\x1a\rsession.proto\x1a\x0bqueue.proto\"\x99\x02\n\rTransferState\x12;\n\x07options\x18\x01 \x01(\x0b\x32*.spotify.player.proto.ContextPlayerOptions\x12\x39\n\x08playback\x18\x02 \x01(\x0b\x32'.spotify.player.proto.transfer.Playback\x12?\n\x0f\x63urrent_session\x18\x03 \x01(\x0b\x32&.spotify.player.proto.transfer.Session\x12\x33\n\x05queue\x18\x04 \x01(\x0b\x32$.spotify.player.proto.transfer.Queue\x12\x1a\n\x12\x63reation_timestamp\x18\x05 \x01(\x03\x42\x18\n\x14\x63om.spotify.transferH\x02",
    dependencies=[
        context__player__options__pb2.DESCRIPTOR,
        playback__pb2.DESCRIPTOR,
        session__pb2.DESCRIPTOR,
        queue__pb2.DESCRIPTOR,
    ],
)

_TRANSFERSTATE = _descriptor.Descriptor(
    name="TransferState",
    full_name="spotify.player.proto.transfer.TransferState",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="options",
            full_name="spotify.player.proto.transfer.TransferState.options",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="playback",
            full_name="spotify.player.proto.transfer.TransferState.playback",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="current_session",
            full_name=
            "spotify.player.proto.transfer.TransferState.current_session",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="queue",
            full_name="spotify.player.proto.transfer.TransferState.queue",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="creation_timestamp",
            full_name=
            "spotify.player.proto.transfer.TransferState.creation_timestamp",
            index=4,
            number=5,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto2",
    extension_ranges=[],
    oneofs=[],
    serialized_start=130,
    serialized_end=411,
)

_TRANSFERSTATE.fields_by_name[
    "options"].message_type = context__player__options__pb2._CONTEXTPLAYEROPTIONS
_TRANSFERSTATE.fields_by_name[
    "playback"].message_type = playback__pb2._PLAYBACK
_TRANSFERSTATE.fields_by_name[
    "current_session"].message_type = session__pb2._SESSION
_TRANSFERSTATE.fields_by_name["queue"].message_type = queue__pb2._QUEUE
DESCRIPTOR.message_types_by_name["TransferState"] = _TRANSFERSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TransferState = _reflection.GeneratedProtocolMessageType(
    "TransferState",
    (_message.Message, ),
    {
        "DESCRIPTOR": _TRANSFERSTATE,
        "__module__": "transfer_state_pb2"
        # @@protoc_insertion_point(class_scope:spotify.player.proto.transfer.TransferState)
    },
)
_sym_db.RegisterMessage(TransferState)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: queue.proto
"""Generated protocol buffer code."""
import ContextTrack_pb2 as context__track__pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name="queue.proto",
    package="spotify.player.proto.transfer",
    syntax="proto2",
    serialized_options=b"\n\024com.spotify.transferH\002",
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\x0bqueue.proto\x12\x1dspotify.player.proto.transfer\x1a\x13\x63ontext_track.proto"U\n\x05Queue\x12\x32\n\x06tracks\x18\x01 \x03(\x0b\x32".spotify.player.proto.ContextTrack\x12\x18\n\x10is_playing_queue\x18\x02 \x01(\x08\x42\x18\n\x14\x63om.spotify.transferH\x02',
    dependencies=[
        context__track__pb2.DESCRIPTOR,
    ],
)

_QUEUE = _descriptor.Descriptor(
    name="Queue",
    full_name="spotify.player.proto.transfer.Queue",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="tracks",
            full_name="spotify.player.proto.transfer.Queue.tracks",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="is_playing_queue",
            full_name="spotify.player.proto.transfer.Queue.is_playing_queue",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto2",
    extension_ranges=[],
    oneofs=[],
    serialized_start=67,
    serialized_end=152,
)

_QUEUE.fields_by_name[
    "tracks"].message_type = context__track__pb2._CONTEXTTRACK
DESCRIPTOR.message_types_by_name["Queue"] = _QUEUE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Queue = _reflection.GeneratedProtocolMessageType(
    "Queue",
    (_message.Message, ),
    {
        "DESCRIPTOR": _QUEUE,
        "__module__": "queue_pb2"
        # @@protoc_insertion_point(class_scope:spotify.player.proto.transfer.Queue)
    },
)
_sym_db.RegisterMessage(Queue)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: spotify/login5/v3/challenges/hashcash.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import \
    duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name="spotify/login5/v3/challenges/hashcash.proto",
    package="spotify.login5.v3.challenges",
    syntax="proto3",
    serialized_options=b"\n\024com.spotify.login5v3",
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n+spotify/login5/v3/challenges/hashcash.proto\x12\x1cspotify.login5.v3.challenges\x1a\x1egoogle/protobuf/duration.proto"3\n\x11HashcashChallenge\x12\x0e\n\x06prefix\x18\x01 \x01(\x0c\x12\x0e\n\x06length\x18\x02 \x01(\x05"O\n\x10HashcashSolution\x12\x0e\n\x06suffix\x18\x01 \x01(\x0c\x12+\n\x08\x64uration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x16\n\x14\x63om.spotify.login5v3b\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,
    ],
)

_HASHCASHCHALLENGE = _descriptor.Descriptor(
    name="HashcashChallenge",
    full_name="spotify.login5.v3.challenges.HashcashChallenge",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="prefix",
            full_name="spotify.login5.v3.challenges.HashcashChallenge.prefix",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="length",
            full_name="spotify.login5.v3.challenges.HashcashChallenge.length",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=109,
    serialized_end=160,
)

_HASHCASHSOLUTION = _descriptor.Descriptor(
    name="HashcashSolution",
    full_name="spotify.login5.v3.challenges.HashcashSolution",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="suffix",
            full_name="spotify.login5.v3.challenges.HashcashSolution.suffix",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="duration",
            full_name="spotify.login5.v3.challenges.HashcashSolution.duration",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=162,
    serialized_end=241,
)

_HASHCASHSOLUTION.fields_by_name[
    "duration"].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name["HashcashChallenge"] = _HASHCASHCHALLENGE
DESCRIPTOR.message_types_by_name["HashcashSolution"] = _HASHCASHSOLUTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HashcashChallenge = _reflection.GeneratedProtocolMessageType(
    "HashcashChallenge",
    (_message.Message, ),
    {
        "DESCRIPTOR": _HASHCASHCHALLENGE,
        "__module__": "spotify.login5.v3.challenges.hashcash_pb2"
        # @@protoc_insertion_point(class_scope:spotify.login5.v3.challenges.HashcashChallenge)
    },
)
_sym_db.RegisterMessage(HashcashChallenge)

HashcashSolution = _reflection.GeneratedProtocolMessageType(
    "HashcashSolution",
    (_message.Message, ),
    {
        "DESCRIPTOR": _HASHCASHSOLUTION,
        "__module__": "spotify.login5.v3.challenges.hashcash_pb2"
        # @@protoc_insertion_point(class_scope:spotify.login5.v3.challenges.HashcashSolution)
    },
)
_sym_db.RegisterMessage(HashcashSolution)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: session.proto
"""Generated protocol buffer code."""
import Context_pb2 as context__pb2
import ContextPlayerOptions_pb2 as context__player__options__pb2
import PlayOrigin_pb2 as play__origin__pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name="session.proto",
    package="spotify.player.proto.transfer",
    syntax="proto2",
    serialized_options=b"\n\024com.spotify.transferH\002",
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\rsession.proto\x12\x1dspotify.player.proto.transfer\x1a\rcontext.proto\x1a\x1c\x63ontext_player_options.proto\x1a\x11play_origin.proto"\xd3\x01\n\x07Session\x12\x35\n\x0bplay_origin\x18\x01 \x01(\x0b\x32 .spotify.player.proto.PlayOrigin\x12.\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\x1d.spotify.player.proto.Context\x12\x13\n\x0b\x63urrent_uid\x18\x03 \x01(\t\x12L\n\x10option_overrides\x18\x04 \x01(\x0b\x32\x32.spotify.player.proto.ContextPlayerOptionOverridesB\x18\n\x14\x63om.spotify.transferH\x02',
    dependencies=[
        context__pb2.DESCRIPTOR,
        context__player__options__pb2.DESCRIPTOR,
        play__origin__pb2.DESCRIPTOR,
    ],
)

_SESSION = _descriptor.Descriptor(
    name="Session",
    full_name="spotify.player.proto.transfer.Session",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="play_origin",
            full_name="spotify.player.proto.transfer.Session.play_origin",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="context",
            full_name="spotify.player.proto.transfer.Session.context",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="current_uid",
            full_name="spotify.player.proto.transfer.Session.current_uid",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="option_overrides",
            full_name="spotify.player.proto.transfer.Session.option_overrides",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto2",
    extension_ranges=[],
    oneofs=[],
    serialized_start=113,
    serialized_end=324,
)

_SESSION.fields_by_name[
    "play_origin"].message_type = play__origin__pb2._PLAYORIGIN
_SESSION.fields_by_name["context"].message_type = context__pb2._CONTEXT
_SESSION.fields_by_name[
    "option_overrides"].message_type = context__player__options__pb2._CONTEXTPLAYEROPTIONOVERRIDES
DESCRIPTOR.message_types_by_name["Session"] = _SESSION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Session = _reflection.GeneratedProtocolMessageType(
    "Session",
    (_message.Message, ),
    {
        "DESCRIPTOR": _SESSION,
        "__module__": "session_pb2"
        # @@protoc_insertion_point(class_scope:spotify.player.proto.transfer.Session)
    },
)
_sym_db.RegisterMessage(Session)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from Cryptodome import Random
from librespot import util
import io
import re
import struct
import typing

if typing.TYPE_CHECKING:
    from librespot.core import Session


class CipherPair:
    __receive_cipher: Shannon
    __receive_nonce = 0
    __send_cipher: Shannon
    __send_nonce = 0

    def __init__(self, send_key: bytes, receive_key: bytes):
        self.__send_cipher = Shannon()
        self.__send_cipher.key(send_key)
        self.__receive_cipher = Shannon()
        self.__receive_cipher.key(receive_key)

    def send_encoded(self, connection: Session.ConnectionHolder, cmd: bytes,
                     payload: bytes) -> None:
        """
        Send decrypted data to the socket
        :param connection:
        :param cmd:
        :param payload:
        :return:
        """
        self.__send_cipher.nonce(self.__send_nonce)
        self.__send_nonce += 1
        buffer = io.BytesIO()
        buffer.write(cmd)
        buffer.write(struct.pack(">H", len(payload)))
        buffer.write(payload)
        buffer.seek(0)
        contents = self.__send_cipher.encrypt(buffer.read())
        mac = self.__send_cipher.finish(4)
        connection.write(contents)
        connection.write(mac)
        connection.flush()

    def receive_encoded(self, connection: Session.ConnectionHolder) -> Packet:
        """
        Receive and parse decrypted data from the socket
        Args:
            connection: ConnectionHolder
        Return:
            The parsed packet will be returned
        """
        try:
            self.__receive_cipher.nonce(self.__receive_nonce)
            self.__receive_nonce += 1
            header_bytes = self.__receive_cipher.decrypt(connection.read(3))
            cmd = struct.pack(">s", bytes([header_bytes[0]]))
            payload_length = (header_bytes[1] << 8) | (header_bytes[2] & 0xff)
            payload_bytes = self.__receive_cipher.decrypt(
                connection.read(payload_length))
            mac = connection.read(4)
            expected_mac = self.__receive_cipher.finish(4)
            if mac != expected_mac:
                raise RuntimeError()
            return Packet(cmd, payload_bytes)
        except IndexError:
            raise RuntimeError("Failed to receive packet")


class DiffieHellman:
    """
    DiffieHellman Keyexchange
    """
    __prime = int.from_bytes(
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xc9\x0f'
        b'\xda\xa2!h\xc24\xc4\xc6b\x8b\x80\xdc\x1c'
        b'\xd1)\x02N\x08\x8ag\xcct\x02\x0b\xbe\xa6;'
        b'\x13\x9b"QJ\x08y\x8e4\x04\xdd\xef\x95\x19'
        b'\xb3\xcd:C\x1b0+\nm\xf2_\x147O\xe15mmQ\xc2'
        b'E\xe4\x85\xb5vb^~\xc6\xf4LB\xe9\xa6:6 \xff'
        b'\xff\xff\xff\xff\xff\xff\xff',
        byteorder="big")
    __private_key: int
    __public_key: int

    def __init__(self):
        key_data = Random.get_random_bytes(0x5f)
        self.__private_key = int.from_bytes(key_data, byteorder="big")
        self.__public_key = pow(2, self.__private_key, self.__prime)

    def compute_shared_key(self, remote_key_bytes: bytes):
        """
        Compute shared_key
        """
        remote_key = int.from_bytes(remote_key_bytes, "big")
        return pow(remote_key, self.__private_key, self.__prime)

    def private_key(self) -> int:
        """
        Return DiffieHellman's private key
        Returns:
            DiffieHellman's private key
        """
        return self.__private_key

    def public_key(self) -> int:
        """
        Return DiffieHellman's public key
        Returns:
            DiffieHellman's public key
        """
        return self.__public_key

    def public_key_bytes(self) -> bytes:
        """
        Return DiffieHellman's packed public key
        Returns:
            DiffieHellman's packed public key
        """
        return util.int_to_bytes(self.__public_key)


class Packet:
    cmd: bytes
    payload: bytes

    def __init__(self, cmd: bytes, payload: bytes):
        self.cmd = cmd
        self.payload = payload

    def is_cmd(self, cmd: bytes) -> bool:
        return cmd == self.cmd

    class Type:
        secret_block = b"\x02"
        ping = b"\x04"
        stream_chunk = b"\x08"
        stream_chunk_res = b"\x09"
        channel_error = b"\x0a"
        channel_abort = b"\x0b"
        request_key = b"\x0c"
        aes_key = b"\x0d"
        aes_key_error = b"\x0e"
        image = b"\x19"
        country_code = b"\x1b"
        pong = b"\x49"
        pong_ack = b"\x4a"
        pause = b"\x4b"
        product_info = b"\x50"
        legacy_welcome = b"\x69"
        license_version = b"\x76"
        login = b"\xab"
        ap_welcome = b"\xac"
        auth_failure = b"\xad"
        mercury_req = b"\xb2"
        mercury_sub = b"\xb3"
        mercury_unsub = b"\xb4"
        mercury_event = b"\xb5"
        track_ended_time = b"\x82"
        unknown_data_all_zeros = b"\x1f"
        preferred_locale = b"\x74"
        unknown_0x4f = b"\x4f"
        unknown_0x0f = b"\x0f"
        unknown_0x10 = b"\x10"

        @staticmethod
        def parse(val: typing.Union[bytes, None]) -> typing.Union[bytes, None]:
            for cmd in [
                    Packet.Type.__dict__[attr] for attr in Packet.Type.__dict__
                    if re.search("__.+?__", attr) is None
                    and type(Packet.Type.__dict__[attr]) is bytes
            ]:
                if cmd == val:
                    return cmd
            return None

        @staticmethod
        def for_method(method: str) -> bytes:
            if method == "SUB":
                return Packet.Type.mercury_sub
            if method == "UNSUB":
                return Packet.Type.mercury_unsub
            return Packet.Type.mercury_req


class Shannon:
    n = 16
    fold = n
    initkonst = 0x6996c53a
    keyp = 13
    r: list
    crc: list
    init_r: list
    konst: int
    sbuf: int
    mbuf: int
    nbuf: int

    def __init__(self):
        self.r = [0 for _ in range(self.n)]
        self.crc = [0 for _ in range(self.n)]
        self.init_r = [0 for _ in range(self.n)]

    def rotl(self, i: int, distance: int) -> int:
        return ((i << distance) | (i >> (32 - distance))) & 0xffffffff

    def sbox(self, i: int) -> int:
        i ^= self.rotl(i, 5) | self.rotl(i, 7)
        i ^= self.rotl(i, 19) | self.rotl(i, 22)
        return i

    def sbox2(self, i: int) -> int:
        i ^= self.rotl(i, 7) | self.rotl(i, 22)
        i ^= self.rotl(i, 5) | self.rotl(i, 19)
        return i

    def cycle(self) -> None:
        t: int
        t = self.r[12] ^ self.r[13] ^ self.konst
        t = self.sbox(t) ^ self.rotl(self.r[0], 1)
        for i in range(1, self.n):
            self.r[i - 1] = self.r[i]
        self.r[self.n - 1] = t
        t = self.sbox2(self.r[2] ^ self.r[15])
        self.r[0] ^= t
        self.sbuf = t ^ self.r[8] ^ self.r[12]

    def crc_func(self, i: int) -> None:
        t: int
        t = self.crc[0] ^ self.crc[2] ^ self.crc[15] ^ i
        for j in range(1, self.n):
            self.crc[j - 1] = self.crc[j]
        self.crc[self.n - 1] = t

    def mac_func(self, i: int) -> None:
        self.crc_func(i)
        self.r[self.keyp] ^= i

    def init_state(self) -> None:
        self.r[0] = 1
        self.r[1] = 1
        for i in range(2, self.n):
            self.r[i] = self.r[i - 1] + self.r[i - 2]
        self.konst = self.initkonst

    def save_state(self) -> None:
        for i in range(self.n):
            self.init_r[i] = self.r[i]

    def reload_state(self) -> None:
        for i in range(self.n):
            self.r[i] = self.init_r[i]

    def gen_konst(self) -> None:
        self.konst = self.r[0]

    def add_key(self, k: int) -> None:
        self.r[self.keyp] ^= k

    def diffuse(self) -> None:
        for _ in range(self.fold):
            self.cycle()

    def load_key(self, key: bytes) -> None:
        i: int
        j: int
        t: int
        padding_size = int((len(key) + 3) / 4) * 4 - len(key)
        key = key + (b"\x00" * padding_size) + struct.pack("<I", len(key))
        for i in range(0, len(key), 4):
            self.r[self.keyp] = \
                self.r[self.keyp] ^ \
                struct.unpack("<I", key[i: i + 4])[0]
            self.cycle()
        for i in range(self.n):
            self.crc[i] = self.r[i]
        self.diffuse()
        for i in range(self.n):
            self.r[i] ^= self.crc[i]

    def key(self, key: bytes) -> None:
        self.init_state()
        self.load_key(key)
        self.gen_konst()
        self.save_state()
        self.nbuf = 0

    def nonce(self, nonce: typing.Union[bytes, int]) -> None:
        if type(nonce) is int:
            nonce = bytes(struct.pack(">I", nonce))
        self.reload_state()
        self.konst = self.initkonst
        self.load_key(nonce)
        self.gen_konst()
        self.nbuf = 0

    def encrypt(self, buffer: bytes, n: int = None) -> bytes:
        if n is None:
            return self.encrypt(buffer, len(buffer))
        buffer = bytearray(buffer)
        i = 0
        j: int
        t: int
        if self.nbuf != 0:
            while self.nbuf != 0 and n != 0:
                self.mbuf ^= (buffer[i] & 0xff) << (32 - self.nbuf)
                buffer[i] ^= (self.sbuf >> (32 - self.nbuf)) & 0xff
                i += 1
                self.nbuf -= 8
                n -= 1
            if self.nbuf != 0:
                return b""
            self.mac_func(self.mbuf)
        j = n & ~0x03
        while i < j:
            self.cycle()
            t = ((buffer[i + 3] & 0xFF) << 24) | \
                ((buffer[i + 2] & 0xFF) << 16) | \
                ((buffer[i + 1] & 0xFF) << 8) | \
                (buffer[i] & 0xFF)
            self.mac_func(t)
            t ^= self.sbuf
            buffer[i + 3] = (t >> 24) & 0xFF
            buffer[i + 2] = (t >> 16) & 0xFF
            buffer[i + 1] = (t >> 8) & 0xFF
            buffer[i] = t & 0xFF
            i += 4
        n &= 0x03
        if n != 0:
            self.cycle()
            self.mbuf = 0
            self.nbuf = 32
            while self.nbuf != 0 and n != 0:
                self.mbuf ^= (buffer[i] & 0xff) << (32 - self.nbuf)
                buffer[i] ^= (self.sbuf >> (32 - self.nbuf)) & 0xff
                i += 1
                self.nbuf -= 8
                n -= 1
        return bytes(buffer)

    def decrypt(self, buffer: bytes, n: int = None) -> bytes:
        if n is None:
            return self.decrypt(buffer, len(buffer))
        buffer = bytearray(buffer)
        i = 0
        j: int
        t: int
        if self.nbuf != 0:
            while self.nbuf != 0 and n != 0:
                buffer[i] ^= (self.sbuf >> (32 - self.nbuf)) & 0xff
                self.mbuf ^= (buffer[i] & 0xff) << (32 - self.nbuf)
                i += 1
                self.nbuf -= 8
                n -= 1
            if self.nbuf != 0:
                return b""
            self.mac_func(self.mbuf)
        j = n & ~0x03
        while i < j:
            self.cycle()
            t = ((buffer[i + 3] & 0xFF) << 24) | \
                ((buffer[i + 2] & 0xFF) << 16) | \
                ((buffer[i + 1] & 0xFF) << 8) | \
                (buffer[i] & 0xFF)
            t ^= self.sbuf
            self.mac_func(t)
            buffer[i + 3] = (t >> 24) & 0xFF
            buffer[i + 2] = (t >> 16) & 0xFF
            buffer[i + 1] = (t >> 8) & 0xFF
            buffer[i] = t & 0xFF
            i += 4
        n &= 0x03
        if n != 0:
            self.cycle()
            self.mbuf = 0
            self.nbuf = 32
            while self.nbuf != 0 and n != 0:
                buffer[i] ^= (self.sbuf >> (32 - self.nbuf)) & 0xff
                self.mbuf ^= (buffer[i] & 0xff) << (32 - self.nbuf)
                i += 1
                self.nbuf -= 8
                n -= 1
        return bytes(buffer)

    def finish(self, n: int) -> bytes:
        buffer = bytearray(4)
        i = 0
        j: int
        if self.nbuf != 0:
            self.mac_func(self.mbuf)
        self.cycle()
        self.add_key(self.initkonst ^ (self.nbuf << 3))
        self.nbuf = 0
        for j in range(self.n):
            self.r[j] ^= self.crc[j]
        self.diffuse()
        while n > 0:
            self.cycle()
            if n >= 4:
                buffer[i + 3] = (self.sbuf >> 24) & 0xff
                buffer[i + 2] = (self.sbuf >> 16) & 0xff
                buffer[i + 1] = (self.sbuf >> 8) & 0xff
                buffer[i] = self.sbuf & 0xff
                n -= 4
                i += 4
            else:
                for j in range(n):
                    buffer[i + j] = (self.sbuf >> (i * 8)) & 0xff
                break
        return bytes(buffer)
